//////////////////////////////////////////////
// NovaWM - Nova Window Manager for X11     //
//////////////////////////////////////////////
// By: Tim Walters                          //
//////////////////////////////////////////////
// Copyright (C) 2001-2002 Tim Walters      //
//////////////////////////////////////////////

////////////////////////////////////////////////////////////////
//This code is released under the terms of the GNU GPL. Refer //
//to the license file included with this source code.         //
////////////////////////////////////////////////////////////////


#ifndef NOVAWM_H
#define NOVAWM_H

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <string.h>
#include <time.h>

//X11 Includes
#include <X11/X.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/keysym.h>
#include <X11/cursorfont.h>
#include <X11/Xatom.h>
#include <X11/Xmd.h>
#include <X11/xpm.h>

#define VERSION "0.6 Alpha"

//NEW - NovaWM_Window_Hints class for NovaWM_Window_Hints support
//NovaTK applications will feature NovaWM_Window_Hints support
class NovaWM_Window_Hints
{
      public:
	int hasTitlebar;
	int hasBorder;
	int titlebarPosition;
	int hasKillButton;
	int hasStretchButton;
	int hasHideButton;
	unsigned int backgroundColor;
	int textStlye;
	unsigned int textColor;
	int noMoving;
	int desktop;
	int x, y, width, height;
};

class Novawm_Window
{
      public:
	Window window;
	Window titlebar;
	Window kill;
	Window stretch;
	Window hide;
	Window *tChild;		//Transient Window
	char *title;
	int x, y, width, height;
	int orig_width, orig_height;
	int state;
	int nTChildren;
	int desktop;
	bool hasTitlebar;
	bool isTransient;
	bool ignoreTransient;
	XSizeHints *size_hints;
	NovaWM_Window_Hints *novawm_window_hints;

	Novawm_Window *prev;
	Novawm_Window *next;

	  Novawm_Window ();
	 ~Novawm_Window ();

	void SetState (int set_state);
	int GetState ();
	void Destroy ();
};

//NovaWM_Hints are for applications to work with NovaWM and provide special
//features. It is used to obtain info from NovaWM and change variables
class NovaWM_Hints
{
      public:
	int currentDesktop;	//Current Desktop
	int nDesktops;		//Number of desktops
	int maxDesktops;	//Maximum number of desktops
	int nWindows;		//Number of managed windows
	Novawm_Window *topWindow;	//First window (For window info)
	unsigned int defaultColor;	//Default Color (for apps to match NovaWM)
};


class NovaWM
{
      public:
	void X11_Init ();
	void WM_Start ();	//Reparent All
	void Shutdown ();
	void Quick_Shutdown ();
	void Restart ();
	void Run (char *command);
	void Set_NovaWM_Hints ();
	Pixmap LoadXPM (char **xpm_data, int width, int height, GC * xpmgc);
	NovaWM_Hints *Get_NovaWM_Hints ();
	void UpdateClock ();
	void CreateBar ();
	void DrawBar ();
	void CreateWinList ();
	void DrawWinList ();
	void DrawArrow (Window xWindow);
	void DrawUpArrow (Window xWindow);
	void CreateNovaMenu ();
	char *GetTime (int format);
	void CheckClock ();
	void HighlightListItem (int cY);
	void DestroyBar ();
	void DestroyWinList ();
	void List_Scroll (int units);
	void novaMenu_Click(int item);

	int bar_x;
	int nSeconds;		//Number of Seconds till minute change
	int lstItems, selItem, topItem;


	time_t oldTime;
	Window NovaBar;
	Window nbClock;
	Window nbCurWin;
	Window nbListButton;
	Window nbWinList;
	Window nbWLstScrllBar;
	Window nbWLstItemWin;
	Window nbWLstScrllUp;
	Window nbWLstScrllDwn;
	Window nbButton;
};

//Stores configuration data and operations
class Config
{
      public:
	int shwFocusMode;	//Show Focus Mode
	int focusType;
	unsigned int ntbColor;
	unsigned int ftbColor;
	int tbTextStyle;
	bool disableBar;
	bool enableRootMenus;
	int clockFormat;
	bool disableBarMenu;
	int barPosition;
	bool disableClock;
	bool enableBorder;
	bool nonStandardMenu;

	int nMenuItems;
	char **itemExec;

	Config();
	~Config();
	
	void SetDefaults ();
	int Load (char *filename);
	void CreateNew ();
	void LoadMenu ();
	char *readLine ();
	char **SplitString (char c, char *str);
	int ParseLine (char *line);
	int IncludeFile (char *file);
};

class WinMgr
{
      public:
	WinMgr ();
	~WinMgr ();
	void NewWindow (Window x11_window);
	Novawm_Window *WindowSearch (Window x11_window);
	void Redraw (Novawm_Window * novawm_window);
	void Move (Novawm_Window * novawm_window);
	void Stretch (Novawm_Window * novawm_window);
	void Raise (Novawm_Window * novawm_window);
	void SetFocus (Novawm_Window * novawm_window);
	void Hide (Novawm_Window * novawm_window);
	void Resize (Novawm_Window * novawm_window);
	void Kill (Novawm_Window * novawm_window);
	void Create_ListMenu (int menu_x, int menu_y);
	void listMenu_Click (int item);
	void SetDesktop (int desktop);
	int AddDesktop ();
	void RemoveDesktop ();
	void SendToDesktop (Novawm_Window * window, int desktop);

	int cDesktop;		//Current Desktop
	int nDesktops;
	int maxDesktops;
	//Novawm_Window **listWindow; //Pointers to items in the list
	int nListWindow;
	int unmapignore;

	Novawm_Window *topWindow;
	Novawm_Window *lastWindow;
	Novawm_Window *focusWindow;
};

class Events
{
      public:
	void NovaWM_EventLoop ();
	void Event_Expose (XExposeEvent * event);
	void Event_ButtonDown (XButtonEvent * event);
	void Event_ButtonUp (XButtonEvent * event);
	void Event_Map (XMapRequestEvent * event);
	void Event_Unmap (XUnmapEvent * event);
	void Event_Motion (XCrossingEvent * event);
	void Event_Destroy (XDestroyWindowEvent * event);
	void Event_Configure (XConfigureRequestEvent * event);
	void Event_Property (XPropertyEvent * Event);
	void Event_ClientMessage (XClientMessageEvent * event);
	void Event_FocusIn (XFocusChangeEvent * event);
	void Event_FocusOut (XFocusChangeEvent * event);
	void Event_EnterWindow (XCrossingEvent * event);
	void Event_LeaveWindow (XCrossingEvent * event);
	void Event_KeyUp (XKeyEvent * event);
	void DrawDown (Window window);
	void DrawUp (Window window);

	void rootMenu_Click (int item);
	void windowMenu_Click (int item);

	Window downWindow;
};

class MotifHints
{
      public:
	unsigned long flags;
	unsigned long functions;
	unsigned long decorations;
};

//NEW - Better menu code!
//NOTE: ?+MENU_SPLIT+? as a item text will be a splitter
class NovaWM_Menu
{
      private:
	int x, y, width, height;
	int nItems;
	int itemHeight;
	char *itemText[255];
	Window x11_window;
	int sel;		//Selected Item
	bool open;
	XFontStruct *font;

      public:
	  NovaWM_Menu ();
	 ~NovaWM_Menu ();
	int Create (int c_x, int c_y);
	void AddItem (char *text);
	int GetSelection ();
	void HighlightItem (int cusor_x, int cursor_y);
	void DrawItems ();
	void DrawSelection ();
	void Clear ();
	bool isOpen ();
	Window menuWindow ();
	void Destroy ();
	void SetItemHeight (int height);
	void SetFont (XFontStruct * xfont);
};

extern NovaWM novawm;
extern WinMgr *winmgr;
extern Events events;
extern Config cfg;
extern Display *display;
extern Window root;
extern int screen;
extern GC novawm_gc;
extern XColor color_grey;
extern XColor color_blue;
extern XColor color_red;
extern XColor color_darkblue;
extern XColor color_navyblue;
extern XFontStruct *novawmFont;
extern XFontStruct *nbFont;
extern Cursor Arrow;
extern Cursor MoveCursor;
extern Cursor ResizeCursor;
extern Cursor ResizeRightCursor;
extern Cursor ResizeDownCursor;
extern Atom stateAtom;
extern Atom deleteAtom;
extern Atom protocolAtom;
extern Atom motifAtom;
extern Atom novawmAtom;
extern Atom takefocusAtom;
extern Atom changestateAtom;
extern Atom gnomeSupportAtom;
extern Atom netSupportAtom;
extern Atom netStateAtom;
extern Window gnomeSupportWindow;
extern Pixmap killPixmap;
extern Pixmap stretchPixmap;
extern Pixmap hidePixmap;
extern GC killIconGC;
extern GC stretchIconGC;
extern GC hideIconGC;
extern NovaWM_Menu rootMenu;
extern NovaWM_Menu windowMenu;
extern NovaWM_Menu listMenu;
extern NovaWM_Menu novaMenu;
extern NovaWM_Hints *novawmHints;
extern int focusType;

#define COLOR_GREY color_grey.pixel
#define COLOR_BLUE color_blue.pixel
#define COLOR_RED  color_red.pixel
#define COLOR_DARKBLUE color_darkblue.pixel
#define COLOR_NAVYBLUE color_navyblue.pixel

#define COLOR_MAIN COLOR_NAVYBLUE

#define NOVABAR_HEIGHT	18

static void
signal_handle (int signal)
{
	switch (signal)
	{
	case SIGTERM:
		printf ("NovaWM: Caught SIGTERM! Shutting down...\n");
		novawm.Shutdown ();
		break;

	case SIGSEGV:
		printf ("NovaWM: Caught SIGSEGV (Segmentation Fault) !! Attempting to quit quickly...\n");
		novawm.Quick_Shutdown ();
		exit (0);
		break;
	}
}

#endif
