/////////////////////////////////////////////
// NovaWM - Nova Window Manager for X11    //
/////////////////////////////////////////////
// By: Tim Walters                         //
/////////////////////////////////////////////
// Copyright (C) 2001-2002 Tim Walters     //
/////////////////////////////////////////////

////////////////////////////////////////////////////////////////
//This code is released under the terms of the GNU GPL. Refer //
//to the license file included with this source code.         //
////////////////////////////////////////////////////////////////


#include "novawm.h"

WinMgr::WinMgr ()
{
  topWindow = 0;
  lastWindow = 0;
  focusWindow = 0;
  cDesktop = 1;
}

WinMgr::~WinMgr ()
{
}

Novawm_Window::Novawm_Window ()
{
  window = 0;
  titlebar = 0;
  kill = 0;
  stretch = 0;
  hide = 0;
  tChild = 0;
//      title = "Unnamed";

  ignoreTransient = false;

  nTChildren = 0;

  desktop = winmgr->cDesktop;

  hasTitlebar = true;
  isTransient = false;

  size_hints = 0;

  prev = 0;
  next = 0;
}

Novawm_Window::~Novawm_Window ()
{
}

void
WinMgr::NewWindow (Window x11_window)
{
#ifdef DEBUG
  printf ("DEBUG: NewWindow()\n");
#endif

  XWindowAttributes win_attrib;
  XSetWindowAttributes set_attrib;
  long dummy;
  char *name;
  unsigned long udummy;
  Atom adummy;
  int idummy;

  MotifHints *motif_hints = NULL;

  Window tranWin = None;

  Novawm_Window *win = new Novawm_Window;

  XGrabServer (display);

  XGetWindowAttributes (display, x11_window, &win_attrib);

  if (win_attrib.override_redirect == true)
    {
#ifdef DEBUG
	    printf("DEBUG: override_redirect == true\n");
#endif
      delete win;
      XUnmapWindow (display, x11_window);
      unmapignore++;
      return;
    }

/*	XTextProperty *tprop;
	
	if(!XGetWMName(display,x11_window,tprop))
	{
#ifdef DEBUG
		printf("DEBUG: XGetWMName returned bad status\n");
#endif
		win->title = "Unknown";
	}
	
	if(!tprop->value||tprop->nitems <= 0)
		win->title = "Unknown";

	if(tprop->value)
		win->title = (char *)tprop->value;
*/
  XFetchName (display, x11_window, &name);

  if (!name)
    win->title = "Unknown";
  else
    win->title = name;

  win->hasTitlebar = true;

  win->x = win_attrib.x;
  win->y = win_attrib.y;
  win->width = win_attrib.width;
  win->height = win_attrib.height;

  if ((win->y < 20)||(win->y < NOVABAR_HEIGHT))
    win->y = NOVABAR_HEIGHT+20;

  win->orig_width = win_attrib.width;
  win->orig_height = win_attrib.height;

  win->nTChildren = 0;

  win->window = x11_window;

  win->size_hints = XAllocSizeHints ();

  XGetWMNormalHints (display, x11_window, win->size_hints, &dummy);

 if (win_attrib.border_width > 0)
    {
      win->width += win_attrib.border_width * 2;
      win->height += win_attrib.border_width * 2;
    }

  if (win->width > XDisplayWidth (display, screen))
    win->width = XDisplayWidth (display, screen);
  if (win->height > XDisplayHeight (display, screen))
    win->height = XDisplayHeight (display, screen);

  XGetTransientForHint (display, win->window, &tranWin);

  if (tranWin)
    win->isTransient = true;
  else
    win->isTransient = false;

  Novawm_Window *parent = winmgr->WindowSearch (tranWin);

  if (parent)
    {
      parent->nTChildren++;
      if (parent->nTChildren == 1)
	parent->tChild = new Window;

      parent->tChild[parent->nTChildren - 1] = x11_window;
    }

  XGetWindowProperty (display, win->window, motifAtom, 0L, 20L, false,
		      motifAtom, &adummy, &idummy, &udummy, &udummy,
		      (unsigned char **) &motif_hints);

  if (motif_hints)
    {
      if ((motif_hints->flags & (1l << 1)) &&
	  (!(motif_hints->decorations & (1l << 0))))
	{

	  if ((motif_hints->decorations & (1l << 3)) == 0)
	    win->hasTitlebar = false;
	  else
	    win->hasTitlebar = true;
	}
    }

  XFree (motif_hints);

  set_attrib.do_not_propagate_mask =
    ButtonPressMask | ButtonReleaseMask | ButtonMotionMask;

  set_attrib.event_mask =
    ExposureMask | ButtonPressMask | ButtonReleaseMask |
    PropertyChangeMask | SubstructureRedirectMask |
    FocusChangeMask | SubstructureNotifyMask |
    EnterWindowMask | LeaveWindowMask;

  set_attrib.background_pixel = COLOR_MAIN;
  //BlackPixel (display, screen);

  set_attrib.override_redirect = false;	//true;

  if (win->hasTitlebar == true)
  {
    win->titlebar =
      XCreateWindow (display, root, win->x, win->y - 20,
		     win->width, win->height + 20, 0,
		     DefaultDepth (display, screen),
		     CopyFromParent, DefaultVisual (display,
						    screen),
		     CWBackPixel | CWEventMask
		     | CWOverrideRedirect | CWDontPropagate, &set_attrib);

    //Create the Kill/Close Button Window
    win->kill = XCreateWindow(display,win->titlebar,win->width-19,3,16,16,
		    0,DefaultDepth(display,screen),CopyFromParent,
		    DefaultVisual(display,screen),CWBackPixel|CWEventMask|
		    CWDontPropagate,&set_attrib);

    //Create the Stretch/Maximize Button Window
    win->stretch = XCreateWindow(display,win->titlebar,win->width-37,3,16,16,
		    0,DefaultDepth(display,screen),CopyFromParent,
		    DefaultVisual(display,screen),CWBackPixel|CWEventMask|
		    CWDontPropagate,&set_attrib);

    //Create the Hide/Minimize Button Window
    win->hide = XCreateWindow(display,win->titlebar,win->width-56,3,16,16,
		    0,DefaultDepth(display,screen),CopyFromParent,
		    DefaultVisual(display,screen),CWBackPixel|CWEventMask|
		    CWDontPropagate,&set_attrib);

  }

  XChangeWindowAttributes (display, win->window,
			   CWDontPropagate, &set_attrib);


  XSelectInput (display, win->window, PropertyChangeMask | FocusChangeMask);

  XAddToSaveSet (display, win->window);

  if (win->hasTitlebar == true)
    XReparentWindow (display, win->window, win->titlebar, 0, 20);

  if (focusType == 1)
  {
    XGrabButton (display, Button1, AnyModifier, win->titlebar, 1,
		 ButtonPressMask, GrabModeSync, GrabModeAsync, None, None);
    
  }

  if (topWindow == 0)
    topWindow = win;

  win->prev = lastWindow;

  if (lastWindow != 0)
    lastWindow->next = win;

  lastWindow = win;

  win->next = 0;
  XMapRaised (display, win->titlebar);
  XMapWindow (display, win->window);
  
  XSetWindowBackgroundPixmap(display,win->kill,killPixmap);
  XClearWindow(display,win->kill);
  XSetWindowBackgroundPixmap(display,win->stretch,stretchPixmap);
  XClearWindow(display,win->stretch);
  XSetWindowBackgroundPixmap(display,win->hide,hidePixmap);
  XClearWindow(display,win->hide);

  //Map the button windows
  XMapWindow(display,win->kill);
  XMapWindow(display,win->stretch);
  XMapWindow(display,win->hide);

  //win->SetState (NormalState);

  //SetFocus (win);

  /*int winState = win->GetState();
  if(winState != NormalState)
  {
	  win->SetState(WithdrawnState);
	  Hide(win);
  }

  else
  {*/
	  SetFocus(win);
	  Redraw (win);
  //}

  XSync (display, true);//false);

  XUngrabServer (display);

  SendToDesktop (win, winmgr->cDesktop);

}

Novawm_Window *
WinMgr::WindowSearch (Window x11_window)
{
  if (!x11_window || x11_window == root
      || x11_window == rootMenu.menuWindow ()
      || x11_window == windowMenu.menuWindow ())
    return 0;

  for (Novawm_Window * SearchWin = topWindow; SearchWin;
       SearchWin = SearchWin->next)
    {
      if (!SearchWin->window)
	printf ("DEBUG: BAD WINDOW?!\n");
      //              return 0;

      if ((SearchWin->window == x11_window) || (SearchWin->titlebar == x11_window)||(SearchWin->stretch == x11_window)||(SearchWin->kill == x11_window)||(SearchWin->hide == x11_window))
	{
	  return SearchWin;
	}
    }

  return 0;
}

void
WinMgr::Redraw (Novawm_Window * novawm_window)
{

  if (novawm_window->title == NULL)
    {
      printf ("BAD TITLE!\n");
      novawm_window->title = "Unknown";
    }

  XClearWindow (display, novawm_window->titlebar);

  XDrawString (display, novawm_window->titlebar, novawm_gc, 4, 14,
	       novawm_window->title, strlen (novawm_window->title));

  //Draw Raised Border for the titlebar and buttons
  //Titlebar (White lines)
  XDrawLine (display, novawm_window->titlebar, novawm_gc, 0, 0, 0, 20);
  XDrawLine (display, novawm_window->titlebar, novawm_gc, 0, 0,
	     novawm_window->width, 0);
  
	if(events.downWindow != novawm_window->kill)
	{
  //Kill Button (White lines)
  XDrawLine (display, novawm_window->kill, novawm_gc, 0, 0, 0, 16);
  XDrawLine (display, novawm_window->kill, novawm_gc, 0, 0,16,0);
	}
	if(events.downWindow != novawm_window->stretch)
	{
  //Stretch Button (White lines)
  XDrawLine (display, novawm_window->stretch, novawm_gc, 0, 0, 0, 16);
  XDrawLine (display, novawm_window->stretch, novawm_gc, 0, 0,16, 0);
	}
	if(events.downWindow != novawm_window->hide)
	{
  //Hide Button (White lines)
  XDrawLine (display, novawm_window->hide, novawm_gc, 0, 0, 0, 16);
  XDrawLine (display, novawm_window->hide, novawm_gc, 0, 0,16, 0);
	}
	
  XSetForeground (display, novawm_gc, BlackPixel (display, screen));

  if (focusWindow == novawm_window)
    XDrawString (display, novawm_window->titlebar, novawm_gc, 3, 13,
		 novawm_window->title, strlen (novawm_window->title));

  //Titlebar (black lines)
  XDrawLine (display, novawm_window->titlebar, novawm_gc, 1, 19,
	     novawm_window->width, 19);
  XDrawLine (display, novawm_window->titlebar, novawm_gc,
	     novawm_window->width - 1, 1, novawm_window->width - 1, 20);
  
	if(events.downWindow != novawm_window->kill)
	{
  //Kill Button (Black lines)
  XDrawLine (display, novawm_window->kill, novawm_gc, 15,1,15,16);
  XDrawLine (display, novawm_window->kill, novawm_gc, 0,15,16,15);
	}
	if(events.downWindow != novawm_window->stretch)
	{
  //Stretch Button (Black lines) 
  XDrawLine (display, novawm_window->stretch, novawm_gc, 15,1,15,16);
  XDrawLine (display, novawm_window->stretch, novawm_gc, 0,15,16,15);
	}
	if(events.downWindow != novawm_window->hide)
	{
  //Hide Button (Black lines)
  XDrawLine (display, novawm_window->hide, novawm_gc, 15,1,15,16);
  XDrawLine (display, novawm_window->hide, novawm_gc, 0,15,16,15);
	}
                  
  
  XSetForeground (display, novawm_gc, WhitePixel (display, screen));
}

void
WinMgr::Move (Novawm_Window * novawm_window)
{
#ifdef DEBUG
  printf ("DEBUG: Move()\n");
#endif

  XEvent event;
  XConfigureEvent configEvent;
  Window wdummy;
  int x, y, winx, winy;
  unsigned int mdummy;
  int idummy;

  XGrabPointer (display, novawm_window->titlebar, false,
		ButtonReleaseMask | PointerMotionMask, GrabModeAsync,
		GrabModeAsync, None, MoveCursor, CurrentTime);
  //XGrabServer (display);
  XQueryPointer (display, novawm_window->titlebar, &wdummy, &wdummy, &x,
		 &y, &winx, &winy, &mdummy);

  configEvent.type = ConfigureNotify;
  configEvent.event = novawm_window->window;
  configEvent.window = novawm_window->window;
  configEvent.width = novawm_window->width;
  configEvent.height = novawm_window->height;
  configEvent.border_width = 0;
  configEvent.above = Below;
  configEvent.override_redirect = false;

  while (1)
    {
      XMaskEvent (display,
		  ButtonPressMask | ButtonReleaseMask |
		  PointerMotionMask, &event);
      switch (event.type)
	{
	  //case Expose: For redrawing window contents as they get erased
	  //  wmevents.RedrawTitlebar(WindowSearch(event.xexpose.window));
	  //  break;
	case MotionNotify:
	  XQueryPointer (display, novawm_window->titlebar,
			 &wdummy, &wdummy, &x, &y, &idummy, &idummy, &mdummy);


	  XMoveWindow (display, novawm_window->titlebar, //x - winx, y - winy);
              event.xmotion.x_root- winx, //event.xmotion.x,
              event.xmotion.y_root- winy);//event.xmotion.y);

	      //printf ("XR%d X%d XY%d Y%d\n", event.xmotion.x_root,
		//  event.xmotion.x, event.xmotion.y_root, event.xmotion.y);

	  novawm_window->x = event.xmotion.x_root-winx;
	  //event.xmotion.x;
	  novawm_window->y = event.xmotion.y_root-winy+20;
	  //event.xmotion.y+20;
	  break;

	case ButtonRelease:
//                      novawm_window->x = x;
//                      novawm_window->y = y;
	  //XUngrabServer (display);
	  XUngrabPointer (display, CurrentTime);

	  //Add 20 for the titlebar
	  //(Incase the window needs its EXACT position)
	  configEvent.x = novawm_window->x;
	  configEvent.y = novawm_window->y;//+20;

	  XSendEvent (display, novawm_window->window, false,
		      StructureNotifyMask, (XEvent *) & configEvent);

	  //Update the Window


	  return;
	  break;

	}

    }

}

void
WinMgr::Stretch (Novawm_Window * novawm_window)
{
#ifdef DEBUG
  printf ("DEBUG: Stretch()\n");
#endif

  XConfigureEvent configEvent;

  configEvent.type = ConfigureNotify;
  configEvent.event = novawm_window->window;
  configEvent.window = novawm_window->window;
  configEvent.x = novawm_window->x;
  configEvent.y = novawm_window->y;
  configEvent.border_width = 0;
  configEvent.above = Below;
  configEvent.override_redirect = false;

  //Check if it is already stretched
  if ((novawm_window->width != novawm_window->orig_width) &&
      (novawm_window->height != novawm_window->orig_height))
    {
      if (novawm_window->hasTitlebar != true)	//Has no titlebar
	{
	  novawm_window->width = novawm_window->orig_width;
	  novawm_window->height = novawm_window->orig_height;

	  configEvent.width = novawm_window->orig_width;
	  configEvent.height = novawm_window->orig_height;

	  XResizeWindow (display, novawm_window->window,
			 novawm_window->orig_width,
			 novawm_window->orig_height);

	  XMoveWindow (display, novawm_window->window,
			  novawm_window->x,
			  novawm_window->y);

	  XSendEvent (display, novawm_window->window, false,
		      StructureNotifyMask, (XEvent *) & configEvent);
	  
	  //Move the titlebar buttons
	  XMoveWindow(display,novawm_window->kill,novawm_window->width-19,3);
	  XMoveWindow(display,novawm_window->stretch,novawm_window->width-37,3);
	  XMoveWindow(display,novawm_window->hide,novawm_window->width-54,3);
	                       
	  
	  return;
	}

      else			//Has a titlebar
	{
	  novawm_window->width = novawm_window->orig_width;
	  novawm_window->height = novawm_window->orig_height;
	  
	  configEvent.width = novawm_window->orig_width;
	  configEvent.height = novawm_window->orig_height;

	  XResizeWindow (display, novawm_window->window,
			 novawm_window->orig_width,
			 novawm_window->orig_height);

	  XResizeWindow (display, novawm_window->titlebar,
			 novawm_window->orig_width,
			 novawm_window->orig_height + 20);

	  XMoveWindow (display,novawm_window->titlebar,
			  novawm_window->x,novawm_window->y-20);

	  XSendEvent (display, novawm_window->window, false,
		      StructureNotifyMask, (XEvent *) & configEvent);

	  //Move the titlebar buttons
	  XMoveWindow(display,novawm_window->kill,novawm_window->width-19,3);
	  XMoveWindow(display,novawm_window->stretch,novawm_window->width-37,3);          XMoveWindow(display,novawm_window->hide,novawm_window->width-54,3);   
	  
	  return;

	}

    }

  //Stretch to fullscreen
  else
    {
      if (novawm_window->hasTitlebar != true)	//Has no titlebar
	{
	}

      else
	{
	  //Commented out to remember the original position
	  //novawm_window->x = 0;
	  //novawm_window->y = 0;
	  novawm_window->orig_width = novawm_window->width;
	  novawm_window->orig_height = novawm_window->height;

	  novawm_window->width = XDisplayWidth (display, screen);
	  novawm_window->height = XDisplayHeight (display, screen) - 20-NOVABAR_HEIGHT;

	  configEvent.width = XDisplayWidth (display, screen);
	  configEvent.height = XDisplayHeight (display, screen) - 20-NOVABAR_HEIGHT;
	  configEvent.x = 0;
	  configEvent.y = NOVABAR_HEIGHT;

	  XMoveWindow (display, novawm_window->titlebar, 0, NOVABAR_HEIGHT);

	  XResizeWindow (display, novawm_window->window,
			 XDisplayWidth (display, screen),
			 XDisplayHeight (display, screen) - 20);

	  XResizeWindow (display, novawm_window->titlebar,
			 XDisplayWidth (display, screen),
			 XDisplayHeight (display, screen));

	  XSendEvent (display, novawm_window->window, false,
			  StructureNotifyMask, (XEvent *) & configEvent);

	  //Move the titlebar buttons
	  XMoveWindow(display,novawm_window->kill,novawm_window->width-19,3);
	  XMoveWindow(display,novawm_window->stretch,novawm_window->width-37,3);
	  XMoveWindow(display,novawm_window->hide,novawm_window->width-54,3);
	  
	  return;

	}
    }
}

void
WinMgr::Raise (Novawm_Window * novawm_window)
{
#ifdef DEBUG
  printf ("DEBUG: Raise()\n");
#endif

  if (!novawm_window)
    return;

  //Check the desktop
  if (novawm_window->desktop != cDesktop)
    SetDesktop (novawm_window->desktop);	//Goto that desktop

  novawm_window->SetState (NormalState);

  if (novawm_window->hasTitlebar != true)
    XMapRaised (display, novawm_window->window);
  else
    {
      XMapRaised (display, novawm_window->titlebar);
      XMapRaised (display, novawm_window->window);
    }

  if (novawm_window->ignoreTransient != true)
    {
      if (novawm_window->nTChildren > 0)
	{
	  for (int i = 0; i < novawm_window->nTChildren; i++)
	    {
	      //Raise any windows transient to this one
	      Raise (WindowSearch (novawm_window->tChild[i]));
	    }
	}
    }

}

void
WinMgr::SetFocus (Novawm_Window * novawm_window)
{
#ifdef DEBUG
  printf ("DEBUG: SetFocus()\n");
#endif

  Raise (novawm_window);

  focusWindow = novawm_window;

//Redraw all titlebars
  for (Novawm_Window * searchWindow = topWindow; searchWindow;
       searchWindow = searchWindow->next)
    {
      Redraw (searchWindow);

    }

  if (!novawm_window)
    return;

  XSetInputFocus (display, novawm_window->window, RevertToNone, CurrentTime);

  novawm.DrawBar();

}

void
WinMgr::Hide (Novawm_Window * novawm_window)
{
#ifdef DEBUG
  printf ("DEBUG: Hide()\n");
#endif

  if (!novawm_window)
    return;

  if (novawm_window->hasTitlebar != true)
    XUnmapWindow (display, novawm_window->window);
  else
    XUnmapWindow (display, novawm_window->titlebar);

  novawm_window->SetState (WithdrawnState);

  if (novawm_window->nTChildren > 0)
    {
      for (int i = 0; i < novawm_window->nTChildren; i++)
	{
	  //Hide any windows transient to this one
	  Hide (WindowSearch (novawm_window->tChild[i]));
	}
    }

  //if(focusWindow == novawm_window && novawm_window->prev)
  //SetFocus(novawm_window->prev);
  SetFocus (0);

  unmapignore++;
  
}

void
WinMgr::Resize (Novawm_Window * novawm_window)
{
#ifdef DEBUG
  printf ("DEBUG: Resize()\n");
#endif

  if(!novawm_window)
	  return;
  
  XEvent event;
  XConfigureEvent configEvent;
  Window wdummy;
  XWindowAttributes win_attrib;
  int x, y, winx, winy;
  unsigned int mdummy;
  int idummy;
  XGrabPointer (display, novawm_window->titlebar, false,
		ButtonReleaseMask | PointerMotionMask,
		GrabModeAsync, GrabModeAsync, None, MoveCursor, CurrentTime);
//      XGrabServer (display);
  XQueryPointer (display, novawm_window->titlebar, &wdummy,
		 &wdummy, &x, &y, &winx, &winy, &mdummy);
  configEvent.type = ConfigureNotify;
  configEvent.event = novawm_window->window;
  configEvent.window = novawm_window->window;
  configEvent.width = novawm_window->width;
  configEvent.height = novawm_window->height;
  configEvent.border_width = 0;
  configEvent.above = Below;
  configEvent.override_redirect = false;

  XGetWindowAttributes (display, novawm_window->window, &win_attrib);

  //I added this because of problems with the wrong values
  //Update size values
  novawm_window->width = win_attrib.width;
  novawm_window->height = win_attrib.height;
  novawm_window->orig_width = win_attrib.width;
  novawm_window->orig_height = win_attrib.height;

  XWarpPointer (display, None, root, 0, 0, 0, 0,
		novawm_window->x + novawm_window->width + 2,
		novawm_window->y + novawm_window->height + 2);
  while (1)
    {
      XMaskEvent (display,
		  ButtonPressMask | ButtonReleaseMask |
		  PointerMotionMask, &event);
      switch (event.type)
	{
	case MotionNotify:
	  XQueryPointer (display, root, &wdummy, &wdummy,
			 &idummy, &idummy, &x, &y, &mdummy);
	  XResizeWindow (display, novawm_window->titlebar,
			 x - novawm_window->x - 2, y - novawm_window->y - 2);
	  XResizeWindow (display, novawm_window->window,
			 x - novawm_window->x - 2, y - novawm_window->y - 26);

	  //Move the titlebar buttons
	  XMoveWindow(display,novawm_window->kill,novawm_window->width-19,3);
          XMoveWindow(display,novawm_window->stretch,novawm_window->width-37,3);
          XMoveWindow(display,novawm_window->hide,novawm_window->width-54,3);
		    
	  
	  novawm_window->width = x - novawm_window->x - 2;
	  novawm_window->height = y - novawm_window->y - 26;
	  novawm_window->orig_width = x - novawm_window->x - 2;
	  novawm_window->orig_height = y - novawm_window->y - 26;
	  break;
	case ButtonRelease:

//                      XUngrabServer (display);
	  XUngrabPointer (display, CurrentTime);
	  configEvent.width = novawm_window->width;
	  configEvent.height = novawm_window->height;
	  XSendEvent (display, novawm_window->window, false,
		      StructureNotifyMask, (XEvent *) & configEvent);

	  XSync (display, false);

	  return;
	  break;
	}

    }


}

void
WinMgr::Kill (Novawm_Window * novawm_window)
{
#ifdef DEBUG
  printf ("DEBUG: Kill()\n");
#endif

  Atom *pCheck = 0;
  XEvent deleteEvent;
  int nProtocols;
  if (!novawm_window)
    return;
  deleteEvent.type = ClientMessage;
  deleteEvent.xclient.window = novawm_window->window;
  deleteEvent.xclient.message_type = protocolAtom;
  deleteEvent.xclient.format = 32;
  deleteEvent.xclient.data.l[0] = deleteAtom;
  deleteEvent.xclient.data.l[1] = CurrentTime;
  int deleted = 0;
  
  Hide(novawm_window);
  
  if (!novawm_window->window)
    {
      XDestroyWindow (display, novawm_window->titlebar);
      novawm_window->Destroy ();
    }

  
  XGetWMProtocols (display, novawm_window->window, &pCheck, &nProtocols);

  if (pCheck)
    {

      for (int i = 0; i < nProtocols; i++)
	{
	  if (pCheck[i] == deleteAtom)
	    {
	      deleted = 1;
	      XSendEvent (display, novawm_window->window, false,
			  NoEventMask, &deleteEvent);
	      //unmapignore++;
	      break;
	    }
	}

      if (pCheck)
	XFree (pCheck);
    }

  if (deleted != 1)		//The Window doesn't support the Delete Protocol
    {
      XKillClient (display, novawm_window->window);	//Safe?
      //unmapignore++;
    }
}

void
WinMgr::Create_ListMenu (int menu_x, int menu_y)
{
  listMenu.Clear ();		//Clear the menu
  if (!topWindow)		//There are no windows to manage
    {
      listMenu.AddItem ("No windows to manage!");
      return;
    }

  nListWindow = 0;
/*	if(listWindow != 0)
	{
		printf("novawm: debug: list menu wasn't cleared! Aborting...\n");
		delete listWindow;
		listWindow = 0;
//		return;
	}
*/
  //listWindow = new Novawm_Window *;
  for (Novawm_Window * currentWindow = topWindow;
       currentWindow; currentWindow = currentWindow->next)
    {
      if (currentWindow->isTransient != true)
	{
	  listMenu.AddItem (currentWindow->title);
	  nListWindow++;
	}
    }

  listMenu.Create (menu_x, menu_y);
}

void
WinMgr::listMenu_Click (int item)
{
  if (item > nListWindow && item < 0)
    return;
  int i = 0;
  for (Novawm_Window * cWindow = topWindow;
       cWindow || i >= nListWindow; cWindow = cWindow->next)
    {
      if (!cWindow)
	return;

      if (cWindow->isTransient != true)
	{
	  if (i == item)
	    {
	      SetFocus (cWindow);
	      break;
	    }

	  i++;
	}
      //listWindow[item]);
    }

  //Clear list
  nListWindow = 0;
//      if(listWindow)
//              delete listWindow;
  //listWindow = 0;
}

void
Novawm_Window::Destroy ()
{
#ifdef DEBUG
  printf ("DEBUG: Novawm_Window::Destroy()\n");
#endif

  XGrabServer(display);
  
  if (focusType == 1)
    XUngrabButton (display, AnyButton, AnyModifier, titlebar);

  XReparentWindow(display,window,root,x,y);
  
  XRemoveFromSaveSet (display, window);

  if(hasTitlebar == true)
	  XDestroyWindow(display,titlebar);

  XSync(display,false);

  XUngrabServer(display);
  
  if (prev)
    prev->next = next;
  if (next)
    next->prev = prev;
  if (winmgr->topWindow == this)
    winmgr->topWindow = prev;
  if (winmgr->lastWindow == this)
    winmgr->lastWindow = prev;
  if (winmgr->focusWindow == this)
    winmgr->SetFocus (prev);
  delete this;
}

int
Novawm_Window::GetState ()
{
#ifdef DEBUG
  printf ("DEBUG: GetState()\n");
#endif

  Atom gState;
  int format;
  unsigned long dummy;
  long *data;
  int get_state = WithdrawnState;

  XGetWindowProperty (display, window, stateAtom, 0L, 2L,
		      false, stateAtom, &gState, &format,
		      &dummy, &dummy, (unsigned char **) &data);
  if (data)
    get_state = *data;

  XFree (data);
  return get_state;
}

void
Novawm_Window::SetState (int set_state)
{
#ifdef DEBUG
  printf ("DEBUG: SetState(%d)\n", set_state);
#endif

  CARD32 data[2];
  state = set_state;
  data[0] = set_state;
  data[1] = None;
  XChangeProperty (display, window, stateAtom, stateAtom,
		   32, PropModeReplace, (unsigned char *) data, 2);

}

void
WinMgr::SetDesktop (int desktop)
{
#ifdef DEBUG
  printf ("DEBUG: SetDesktop(%d)\n", desktop);
#endif

  if ((desktop > maxDesktops) || (desktop < 1))
    {
#ifdef DEBUG
      printf
	("DEBUG: Attempt to goto desktop outside of allowed desktops.\n");
#endif
      return;			//Bad Desktop
    }

  if (desktop > nDesktops)
    AddDesktop ();

  cDesktop = desktop;

  for (Novawm_Window * window = topWindow; (window)/*||(window=lastWindow)*/; window = window->next)
    {
      if (!window)
	  {
		  #ifdef DEBUG
		  printf("DEBUG: bad window detected\n");
		  #endif
		return;
	  }

      if ((window->desktop != cDesktop) && (window->state == NormalState))
	  {
	  //Don't Hide() the window or it won't show up when we return to its desktop
	  if (window->hasTitlebar == true)
	    XUnmapWindow (display, window->titlebar);
	  else
	    XUnmapWindow (display, window->window);

	  //Don't let the window change its state! (fixes desktop bugs!! :) )
	  unmapignore++;
	  
	  }

	  if ((window->desktop == cDesktop) && (window->state == NormalState))
	  {
	  //Don't Raise() the window because if it's already hidden it shouldn't be mapped
		  Raise(window);
	  /*if (window->hasTitlebar == true)
	    XMapWindow (display, window->titlebar);
	  else
	    XMapWindow (display, window->window);*/
	  }
    }

}

//There may be a more code in AddDesktop() in some other version
int
WinMgr::AddDesktop ()
{
#ifdef DEBUG
  printf ("DEBUG: AddDesktop()\n");
#endif

  nDesktops++;

  return 1;
}

void
WinMgr::RemoveDesktop ()
{
	#ifdef DEBUG
	printf("DEBUG: RemoveDesktop()\n");
	#endif
	
	nDesktops--;
}

void
WinMgr::SendToDesktop (Novawm_Window * window, int desktop)
{
#ifdef DEBUG
  printf ("DEBUG: SendToDesktop(%d)\n",desktop);
#endif

  if ((desktop > maxDesktops) || (desktop < 1))
    {
#ifdef DEBUG
      printf ("DEBUG: Attempt to send a window to a illegal desktop\n");
#endif
      return;
    }

  if(desktop > nDesktops)
	  AddDesktop();

  window->desktop = desktop;

  if ((cDesktop != desktop) && (window->state == NormalState))
    {
      if (window->hasTitlebar == true)
		XUnmapWindow (display, window->titlebar);
      else
		XUnmapWindow (display, window->window);

      //Don't let the window change its state!
      unmapignore++;
      
    }

}
